﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Input.Manipulations;
using System.Windows.Threading;

namespace Silverlight3Manipulation
{
	public partial class MainPage : UserControl
	{
        // manipulation and inertia processors
        private readonly ManipulationProcessor2D manipulationProcessor;
        private readonly InertiaProcessor2D inertiaProcessor;
        private readonly DispatcherTimer inertiaTimer;

        private const float Deceleration = 10f * 96.0f / (1000f * 1000f);
        private const float MinimumFlickVelocity = 2f * 96.0f / 1000f; // 2 inches/s
		private const float MaximumFlickVelocityFactor = 15f;

		public MainPage()
		{
			InitializeComponent();

			Touch.FrameReported += Touch_FrameReported;

            this.manipulationProcessor = new ManipulationProcessor2D(Manipulations2D.All);
            this.manipulationProcessor.Started += OnManipulationStarted;
            this.manipulationProcessor.Delta += OnManipulationDelta;
            this.manipulationProcessor.Completed += OnManipulationCompleted;

			rotate.CenterX = rectangle.Width/2;
			rotate.CenterY = rectangle.Height/2;

            this.inertiaProcessor = new InertiaProcessor2D();
            this.inertiaProcessor.TranslationBehavior.DesiredDeceleration = Deceleration;
//            this.inertiaProcessor.RotationBehavior.DesiredDeceleration = AngularDeceleration;
//            this.inertiaProcessor.ExpansionBehavior.DesiredDeceleration = ExpansionDeceleration;
            this.inertiaProcessor.Delta += OnInertiaDelta;
            this.inertiaProcessor.Completed += OnInertiaCompleted;

			this.inertiaTimer = new DispatcherTimer();
			this.inertiaTimer.Interval = TimeSpan.FromMilliseconds(30);
			this.inertiaTimer.Tick += OnTimerTick;
		}

		void Touch_FrameReported(object sender, TouchFrameEventArgs e)
		{
			TouchPointCollection tpc = e.GetTouchPoints(TouchSurface);
			List<Manipulator2D> manipulators = new List<Manipulator2D>();

			foreach(TouchPoint tp in tpc)
			{
				touchStatus.Text = "Touch " + tp.Action;

				if(tp.Action != TouchAction.Up)
					manipulators.Add(new Manipulator2D(tp.TouchDevice.Id, (float)tp.Position.X, (float)tp.Position.Y));
			}

			manipulationProcessor.ProcessManipulators(DateTime.UtcNow.Ticks, manipulators);
		}

		private void OnManipulationStarted(object sender, Manipulation2DStartedEventArgs e)
		{
			manipStatus.Text = "Manipulation started";
            if (this.inertiaProcessor.IsRunning)
                this.inertiaProcessor.Complete(DateTime.Now.Ticks);
		}

		private void OnManipulationDelta(object sender, Manipulation2DDeltaEventArgs e)
		{
			manipStatus.Text = "Manipulation delta";
			// kind of a cheat
            Canvas.SetLeft(rectangle, Canvas.GetLeft(rectangle) + e.Delta.TranslationX);
            Canvas.SetTop(rectangle, Canvas.GetTop(rectangle) + e.Delta.TranslationY);

            scale.ScaleX = scale.ScaleX + e.Delta.ScaleX - 1;
            scale.ScaleY = scale.ScaleY + e.Delta.ScaleY - 1;

            rotate.Angle += e.Delta.Rotation * 180.0 / Math.PI;
		}

		private void OnManipulationCompleted(object sender, Manipulation2DCompletedEventArgs e)
		{
			manipStatus.Text = "Manipulation completed";
            bool startFlick = false;
            inertiaTimer.Stop();

            if (inertiaProcessor.IsRunning)
                return;

            inertiaProcessor.InitialOriginX = (float)(rectangle.Width / 2);
            inertiaProcessor.InitialOriginY = (float)(rectangle.Height / 2);

			// get the length of our velocity vector
            Vector initialVelocity = new Vector(e.Velocities.LinearVelocityX, e.Velocities.LinearVelocityY);
            double velocityLengthSquared = initialVelocity.LengthSquared;

			// if we're performing a "flick" (i.e. we moved our finger faster than the minimum stated)
            if (velocityLengthSquared > MinimumFlickVelocity * MinimumFlickVelocity)
            {
				// set the initial velocity based on our finger speed
                const double maximumLengthSquared = MaximumFlickVelocityFactor * MinimumFlickVelocity * MinimumFlickVelocity;

                if (velocityLengthSquared > maximumLengthSquared)
                    initialVelocity = Math.Sqrt(maximumLengthSquared / velocityLengthSquared) * initialVelocity;

				// start a flick
                startFlick = true;
                inertiaProcessor.TranslationBehavior.InitialVelocityX = (float)initialVelocity.X;
                inertiaProcessor.TranslationBehavior.InitialVelocityY = (float)initialVelocity.Y;
            }
            else
            {
				// no flick
                inertiaProcessor.TranslationBehavior.InitialVelocityX = 0.0f;
                inertiaProcessor.TranslationBehavior.InitialVelocityY = 0.0f;
            }

			// start the inertia timer
            if (startFlick)
                this.inertiaTimer.Start();
		}

		private void OnInertiaDelta(object sender, Manipulation2DDeltaEventArgs e)
		{
			inertiaStatus.Text = "Inertia delta";
            OnManipulationDelta(sender, e);
		}

		private void OnInertiaCompleted(object sender, Manipulation2DCompletedEventArgs e)
		{
			inertiaStatus.Text = "Inertia completed";
            inertiaTimer.Stop();
		}

		private void OnTimerTick(object sender, EventArgs e)
		{
            this.inertiaProcessor.Process(DateTime.Now.Ticks);
		}
	}
}
